﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;

namespace Platform.ComponentModel
{
    /// <summary>
    /// A manual disposable and observable base class
    /// </summary>
    public class DisposableAndObservableBase : ObservableBase, IDisposable
    {
        private bool _IsDisposed;

        /// <summary>
        /// Garbage collected
        /// </summary>
        ~DisposableAndObservableBase()
        {
            Dispose(false);
        }

        /// <summary>
        /// Manual dispose object
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            _IsDisposed = true;
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// returns true if object is disposed
        /// </summary>
        public bool IsDisposed
        {
            get { return _IsDisposed; }
        }

        /// <summary>
        /// Process disposing
        /// </summary>
        /// <param name="disposing"></param>
        protected virtual void Dispose( bool disposing )
        {
        }
    }
}
