﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Threading;

namespace Platform.Extensions
{
    public static class RxExtensions
    {
        public static async Task<bool> WhenAnyResult<TResult>( this IEnumerable<Task<TResult>> tasks, Func<TResult,bool> condition, CancellationTokenSource cancelation )
        {
            var remainingTasks = new HashSet<Task<TResult>>(tasks);

            // default to unsuccessfull
            bool success = false;

            try
            {
                // wait for completed task while we have tasks
                while (remainingTasks.Any() && !success)
                {
                    // wait for a ready task
                    var readyTask = await Task.WhenAny(remainingTasks);

                    // remove ready task
                    remainingTasks.Remove(readyTask);

                    // check if task ready
                    if (condition(readyTask.Result))
                        success = true;
                }

                // cancel remaining tasks
                cancelation.Cancel();

                // wait until all tasks completed
                Task.WaitAll(remainingTasks.ToArray());
            }
            catch( TaskCanceledException )
            {
                DebugExtensions.WriteLine("Task[] WhenAnyResult was Canceled with result {0}", success);
            }

            // result result
            return success;
        }
    }
}
