﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;

namespace Platform.Extensions
{
    public static class SocketExtensions
    {
        /// <summary>
        /// Send on socket with timeout
        /// </summary>
        /// <param name="socket"></param>
        /// <param name="buffer"></param>
        /// <param name="offset"></param>
        /// <param name="size"></param>
        /// <param name="flags"></param>
        /// <param name="endpoint"></param>
        /// <param name="timeout"></param>
        /// <returns></returns>
        public static int SendTo( this Socket socket, byte[] buffer, int offset, int size, SocketFlags flags, EndPoint endpoint, int timeout  )
        {
            if (timeout != -1 && !socket.Poll(timeout * 1000, SelectMode.SelectWrite))
                throw new TimeoutException("Timeout sending socket data");
            return socket.SendTo(buffer, offset, size, flags, endpoint);
        }

        /// <summary>
        /// Receive on socket with timeout
        /// </summary>
        /// <param name="socket"></param>
        /// <param name="buffer"></param>
        /// <param name="offset"></param>
        /// <param name="size"></param>
        /// <param name="flags"></param>
        /// <param name="endpoint"></param>
        /// <param name="timeout"></param>
        /// <returns></returns>
        public static int ReceiveFrom(this Socket socket, byte[] buffer, int offset, int size, SocketFlags flags, ref EndPoint endpoint, int timeout )
        {
            if (timeout != -1 && !socket.Poll(timeout * 1000, SelectMode.SelectRead))
                throw new TimeoutException("Timeout receiving socket data");
            return socket.ReceiveFrom( buffer, offset, size, flags, ref endpoint);
        }
    }
}
