﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Platform.Extensions
{
    public static class TypeExtensions
    {
        /// <summary>
        /// Returns true if this is a simple convertable type
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        public static bool IsSimpleType(this Type type)
        {
            // primitive
            if (type.IsPrimitive) return true;

            // enum
            if (type.IsEnum) return true;

            // other simple types
            var simple = new Type[] { typeof(String), typeof(Decimal), typeof(DateTime), typeof(DateTimeOffset), typeof(TimeSpan), typeof(Guid) };
            if (simple.Contains(type)) return true;

            // nullable
            if (type.IsGenericType && type.GetGenericTypeDefinition() == typeof(Nullable<>) && IsSimpleType(type.GetGenericArguments()[0]))
                return true;

            // check 
            if (Convert.GetTypeCode(type) != TypeCode.Object)
                return true;

            // otherwise is a complex type
            return false;
        }
    }
}
