﻿using Platform.Extensions;

namespace Platform.Net
{
    /// <summary>
    /// Base class for a parameter 
    /// consisting of a name and a value
    /// </summary>
    public class NetParameter : NetValue
    {
        #region Construction & Fields

        private string _Name;
        private string _Value;

        protected NetParameter()
        {
        }

        protected NetParameter(string source)
            : base(source)
        {
        }

        protected NetParameter(NetStringReader reader)
            : base(reader)
        {
        }

        protected NetParameter(string name, string value)
        {
            _Name = name;
            _Value = value;
        }

        #endregion

        #region Properties

        public string Name
        {
            get { Decode(); return _Name; }
            set { Decode(); _Name = value; }
        }

        public string Value
        {
            get { Decode(); return _Value; }
            set { Decode(); _Value = value; }
        }

        #endregion

        #region Encoding / Decoding

        /// <summary>
        /// Decode Name
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        protected virtual string ReadName( NetStringReader reader )
        {
            return reader.ReadAllOrUntil('=');
        }

        /// <summary>
        /// Decode Value
        /// </summary>
        /// <param name="value"></param>
        protected virtual string ReadValue( NetStringReader reader )
        {
            return reader.ReadAll();
        }

        /// <summary>
        /// Decode Name
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        protected virtual void WriteName( NetStringWriter writer, string name )
        {
            writer.Write(name);
        }

        /// <summary>
        /// Decode Value
        /// </summary>
        /// <param name="value"></param>
        protected virtual void WriteValue( NetStringWriter writer, string value )
        {
            writer.Write(value);
        }

        /// <summary>
        /// Decode name and value
        /// </summary>
        /// <param name="source"></param>
        protected override void OnDecode( NetStringReader reader )
        {
            _Name = ReadName(reader);
            if (!reader.Read('=')) _Value = null;
            else _Value = ReadValue( reader );
        }

        /// <summary>
        /// Convert to Uri string
        /// </summary>
        /// <returns></returns>
        protected override void OnEncode( NetStringWriter writer )
        {
            if (_Name == null) return;
            WriteName( writer, _Name );
            if (_Value == null) return;
            writer.Write('=');
            WriteValue( writer, _Value );
        }

        #endregion

        #region Matching

        /// <summary>
        /// Comparison
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        public override bool IsMatch(NetValue value)
        {
            if (value.GetType() != GetType()) return false;
            var other = value as NetParameter;
            if (!other.Name.IsEqualTo(Name, true)) return false;
            if (!other.Value.IsEqualTo(Value, true)) return false;
            return true;
        }

        #endregion
    }
}
